from typing import List, Optional

import grpc

from tigrisdb.client import TigrisClient
from tigrisdb.errors import TigrisServerError
from tigrisdb.types import ClientConfig
from tigrisdb.types.filters import Filter
from tigrisdb.types.search import DocStatus, IndexedDoc, Query, Result, VectorField
from tigrisdb.types.vector import Document, DocWithScore


class VectorStore:
    def __init__(self, name: str, config: Optional[ClientConfig] = None):
        self.client = TigrisClient(config).get_search()
        self._index_name = name
        self.index = self.client.get_index(name)

    @property
    def name(self):
        return self._index_name

    def create_index(self, dimension: int):
        self.client.create_or_update_index(
            name=self.name,
            schema={
                "title": self.name,
                "additionalProperties": False,
                "type": "object",
                "properties": {
                    "id": {"type": "string"},
                    "text": {"type": "string"},
                    "metadata": {"type": "object"},
                    "embeddings": {
                        "type": "array",
                        "format": "vector",
                        "dimensions": dimension,
                    },
                },
            },
        )

    def add_documents(self, docs: List[Document]) -> List[DocStatus]:
        """Adds documents to index, if the index does not exist, create it. A `Document`
        is a dictionary with following structure:

        ```
        {
            "id": "optional id of a document",
            "text": "Actual content to store",
            "embeddings": "list of float values",
            "metadata": "optional metadata as dict"
        }
        ```

        - `id` is optional and automatically generated once documents are added to index
        - If `id` is given, any existing documents with matching `id` are replaced

        :param docs: list of documents to add to index
        :type docs: list[Document]
        :raises TigrisServerError: thrown i
        :return: List of `ids` for the added documents
        :rtype: list[DocStatus]
        """
        try:
            return self.index.create_many(docs)
        except TigrisServerError as e:
            if (
                e.code == grpc.StatusCode.NOT_FOUND
                and "search index not found" in e.details
            ):
                first_embedding = docs[0]["embeddings"] if docs else []
                inferred_dim = len(first_embedding) if first_embedding else 16
                self.create_index(inferred_dim)
                return self.index.create_many(docs)
            else:
                raise e

    def delete_documents(self, ids: List[str]) -> List[DocStatus]:
        """Delete documents from index.

        :param ids: list of document ids to delete
        :type ids: list[str]
        :return: `ids` of documents and deletion status for each
        :rtype: list[DocStatus]
        """
        return self.index.delete_many(ids)

    def get_documents(self, ids: List[str]) -> List[IndexedDoc]:
        """Retrieve documents from index. It will only have document `ids` found in the
        index.

        :param ids: list of document ids to retrieve
        :type ids: list[str]
        :return: list of documents and associated metadata
        :rtype: list[IndexedDoc]
        """
        return self.index.get_many(ids)

    def similarity_search(
        self, vector: List[float], k: int = 10, filter_by: Optional[Filter] = None
    ) -> List[DocWithScore]:
        """Perform a similarity search and returns documents most similar to the given
        vector with distance.

        :param vector: Search for documents closest to this vector
        :type vector: list[float]
        :param k: number of documents to return, defaults to 10
        :type k: int, optional
        :param filter_by: apply the filter to metadata to only return a subset of
                documents, defaults to None
        :type filter_by: Filter, optional
        :return: list of documents with similarity score (distance from given vector)
        :rtype: list[DocWithScore]
        """
        q = Query(
            vector_query=VectorField("embeddings", vector),
            filter_by=filter_by,
            hits_per_page=k,
        )
        r = self.search(q)
        return [DocWithScore(_h=hit) for hit in r.hits]

    def search(self, query: Query) -> Result:
        return self.index.search(query=query)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: server/v1/observability.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ...google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from ...openapiv3 import annotations_pb2 as openapiv3_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dserver/v1/observability.proto\x12\x1btigrisdata.observability.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x1bopenapiv3/annotations.proto\"\x1a\n\tRetryInfo\x12\r\n\x05\x64\x65lay\x18\x03 \x01(\x05\"d\n\x0c\x45rrorDetails\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x35\n\x05retry\x18\x03 \x01(\x0b\x32&.tigrisdata.observability.v1.RetryInfo\"I\n\x05\x45rror\x12/\n\x04\x63ode\x18\x01 \x01(\x0e\x32!.tigrisdata.observability.v1.Code\x12\x0f\n\x07message\x18\x02 \x01(\t\"\xdc\x03\n\x1dQueryTimeSeriesMetricsRequest\x12\n\n\x02\x64\x62\x18\x01 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x02 \x01(\t\x12\x12\n\ncollection\x18\x03 \x01(\t\x12\x0c\n\x04\x66rom\x18\x04 \x01(\x03\x12\n\n\x02to\x18\x05 \x01(\x03\x12\x13\n\x0bmetric_name\x18\x06 \x01(\t\x12\x46\n\x10tigris_operation\x18\x07 \x01(\x0e\x32,.tigrisdata.observability.v1.TigrisOperation\x12S\n\x11space_aggregation\x18\x08 \x01(\x0e\x32\x38.tigrisdata.observability.v1.MetricQuerySpaceAggregation\x12\x1b\n\x13space_aggregated_by\x18\t \x03(\t\x12\x42\n\x08\x66unction\x18\n \x01(\x0e\x32\x30.tigrisdata.observability.v1.MetricQueryFunction\x12\x10\n\x08quantile\x18\x0b \x01(\x02\x12L\n\x13\x61\x64\x64itionalFunctions\x18\x0c \x03(\x0b\x32/.tigrisdata.observability.v1.AdditionalFunction\"a\n\x12\x41\x64\x64itionalFunction\x12@\n\x06rollup\x18\x01 \x01(\x0b\x32+.tigrisdata.observability.v1.RollupFunctionH\x00\x88\x01\x01\x42\t\n\x07_rollup\"e\n\x0eRollupFunction\x12\x41\n\naggregator\x18\x01 \x01(\x0e\x32-.tigrisdata.observability.v1.RollupAggregator\x12\x10\n\x08interval\x18\x02 \x01(\x03\"\x84\x01\n\x1eQueryTimeSeriesMetricsResponse\x12\x0c\n\x04\x66rom\x18\x01 \x01(\x03\x12\n\n\x02to\x18\x02 \x01(\x03\x12\r\n\x05query\x18\x03 \x01(\t\x12\x39\n\x06series\x18\x04 \x03(\x0b\x32).tigrisdata.observability.v1.MetricSeries\"\x83\x01\n\x0cMetricSeries\x12\x0c\n\x04\x66rom\x18\x01 \x01(\x03\x12\n\n\x02to\x18\x02 \x01(\x03\x12\x0e\n\x06metric\x18\x03 \x01(\t\x12\r\n\x05scope\x18\x04 \x01(\t\x12:\n\ndataPoints\x18\x05 \x03(\x0b\x32&.tigrisdata.observability.v1.DataPoint\"-\n\tDataPoint\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12\r\n\x05value\x18\x02 \x01(\x01\"\x14\n\x12QuotaLimitsRequest\"Q\n\x13QuotaLimitsResponse\x12\x11\n\tReadUnits\x18\x01 \x01(\x03\x12\x12\n\nWriteUnits\x18\x02 \x01(\x03\x12\x13\n\x0bStorageSize\x18\x03 \x01(\x03\"\x13\n\x11QuotaUsageRequest\"\xa7\x01\n\x12QuotaUsageResponse\x12\x11\n\tReadUnits\x18\x01 \x01(\x03\x12\x12\n\nWriteUnits\x18\x02 \x01(\x03\x12\x13\n\x0bStorageSize\x18\x03 \x01(\x03\x12\x1a\n\x12ReadUnitsThrottled\x18\x04 \x01(\x03\x12\x1b\n\x13WriteUnitsThrottled\x18\x05 \x01(\x03\x12\x1c\n\x14StorageSizeThrottled\x18\x06 \x01(\x03\"\x10\n\x0eGetInfoRequest\"\\\n\x0fGetInfoResponse\x12\x16\n\x0eserver_version\x18\x01 \x01(\t\x12\x31\n\x05\x65rror\x18\x02 \x01(\x0b\x32\".tigrisdata.observability.v1.Error*\x85\x03\n\x04\x43ode\x12\x06\n\x02OK\x10\x00\x12\r\n\tCANCELLED\x10\x01\x12\x0b\n\x07UNKNOWN\x10\x02\x12\x14\n\x10INVALID_ARGUMENT\x10\x03\x12\x15\n\x11\x44\x45\x41\x44LINE_EXCEEDED\x10\x04\x12\r\n\tNOT_FOUND\x10\x05\x12\x12\n\x0e\x41LREADY_EXISTS\x10\x06\x12\x15\n\x11PERMISSION_DENIED\x10\x07\x12\x16\n\x12RESOURCE_EXHAUSTED\x10\x08\x12\x17\n\x13\x46\x41ILED_PRECONDITION\x10\t\x12\x0b\n\x07\x41\x42ORTED\x10\n\x12\x10\n\x0cOUT_OF_RANGE\x10\x0b\x12\x11\n\rUNIMPLEMENTED\x10\x0c\x12\x0c\n\x08INTERNAL\x10\r\x12\x0f\n\x0bUNAVAILABLE\x10\x0e\x12\r\n\tDATA_LOSS\x10\x0f\x12\x13\n\x0fUNAUTHENTICATED\x10\x10\x12\x0c\n\x08\x43ONFLICT\x10\x11\x12\x0f\n\x0b\x42\x41\x44_GATEWAY\x10\x12\x12\x16\n\x12METHOD_NOT_ALLOWED\x10\x13\x12\x15\n\x11\x43ONTENT_TOO_LARGE\x10\x14*=\n\x0fTigrisOperation\x12\x07\n\x03\x41LL\x10\x00\x12\x08\n\x04READ\x10\x01\x12\t\n\x05WRITE\x10\x02\x12\x0c\n\x08METADATA\x10\x03*4\n\x13MetricQueryFunction\x12\x08\n\x04RATE\x10\x00\x12\t\n\x05\x43OUNT\x10\x01\x12\x08\n\x04NONE\x10\x02*\x9b\x01\n\x10RollupAggregator\x12\x19\n\x15ROLLUP_AGGREGATOR_SUM\x10\x00\x12\x1b\n\x17ROLLUP_AGGREGATOR_COUNT\x10\x01\x12\x19\n\x15ROLLUP_AGGREGATOR_MIN\x10\x02\x12\x19\n\x15ROLLUP_AGGREGATOR_MAX\x10\x03\x12\x19\n\x15ROLLUP_AGGREGATOR_AVG\x10\x04*A\n\x1bMetricQuerySpaceAggregation\x12\x07\n\x03\x41VG\x10\x00\x12\x07\n\x03MIN\x10\x01\x12\x07\n\x03MAX\x10\x02\x12\x07\n\x03SUM\x10\x03\x32\xeb\x06\n\rObservability\x12\xf7\x01\n\x16QueryTimeSeriesMetrics\x12:.tigrisdata.observability.v1.QueryTimeSeriesMetricsRequest\x1a;.tigrisdata.observability.v1.QueryTimeSeriesMetricsResponse\"d\x82\xd3\xe4\x93\x02/\"*/v1/observability/metrics/timeseries/query:\x01*\xbaG,\n\rObservability\x12\x1bQueries time series metrics\x12\xd5\x01\n\x0bQuotaLimits\x12/.tigrisdata.observability.v1.QuotaLimitsRequest\x1a\x30.tigrisdata.observability.v1.QuotaLimitsResponse\"c\x82\xd3\xe4\x93\x02#\"\x1e/v1/observability/quota/limits:\x01*\xbaG7\n\rObservability\x12&Queries current namespace quota limits\x12\xd0\x01\n\nQuotaUsage\x12..tigrisdata.observability.v1.QuotaUsageRequest\x1a/.tigrisdata.observability.v1.QuotaUsageResponse\"a\x82\xd3\xe4\x93\x02\"\"\x1d/v1/observability/quota/usage:\x01*\xbaG6\n\rObservability\x12%Queries current namespace quota usage\x12\xb4\x01\n\x07GetInfo\x12+.tigrisdata.observability.v1.GetInfoRequest\x1a,.tigrisdata.observability.v1.GetInfoResponse\"N\x82\xd3\xe4\x93\x02\x18\x12\x16/v1/observability/info\xbaG-\n\rObservability\x12\x1cInformation about the serverBA\n\x1d\x63om.tigrisdata.db.api.v1.grpcZ github.com/tigrisdata/tigris/apib\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'server.v1.observability_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\035com.tigrisdata.db.api.v1.grpcZ github.com/tigrisdata/tigris/api'
  _OBSERVABILITY.methods_by_name['QueryTimeSeriesMetrics']._options = None
  _OBSERVABILITY.methods_by_name['QueryTimeSeriesMetrics']._serialized_options = b'\202\323\344\223\002/\"*/v1/observability/metrics/timeseries/query:\001*\272G,\n\rObservability\022\033Queries time series metrics'
  _OBSERVABILITY.methods_by_name['QuotaLimits']._options = None
  _OBSERVABILITY.methods_by_name['QuotaLimits']._serialized_options = b'\202\323\344\223\002#\"\036/v1/observability/quota/limits:\001*\272G7\n\rObservability\022&Queries current namespace quota limits'
  _OBSERVABILITY.methods_by_name['QuotaUsage']._options = None
  _OBSERVABILITY.methods_by_name['QuotaUsage']._serialized_options = b'\202\323\344\223\002\"\"\035/v1/observability/quota/usage:\001*\272G6\n\rObservability\022%Queries current namespace quota usage'
  _OBSERVABILITY.methods_by_name['GetInfo']._options = None
  _OBSERVABILITY.methods_by_name['GetInfo']._serialized_options = b'\202\323\344\223\002\030\022\026/v1/observability/info\272G-\n\rObservability\022\034Information about the server'
  _CODE._serialized_start=1732
  _CODE._serialized_end=2121
  _TIGRISOPERATION._serialized_start=2123
  _TIGRISOPERATION._serialized_end=2184
  _METRICQUERYFUNCTION._serialized_start=2186
  _METRICQUERYFUNCTION._serialized_end=2238
  _ROLLUPAGGREGATOR._serialized_start=2241
  _ROLLUPAGGREGATOR._serialized_end=2396
  _METRICQUERYSPACEAGGREGATION._serialized_start=2398
  _METRICQUERYSPACEAGGREGATION._serialized_end=2463
  _RETRYINFO._serialized_start=121
  _RETRYINFO._serialized_end=147
  _ERRORDETAILS._serialized_start=149
  _ERRORDETAILS._serialized_end=249
  _ERROR._serialized_start=251
  _ERROR._serialized_end=324
  _QUERYTIMESERIESMETRICSREQUEST._serialized_start=327
  _QUERYTIMESERIESMETRICSREQUEST._serialized_end=803
  _ADDITIONALFUNCTION._serialized_start=805
  _ADDITIONALFUNCTION._serialized_end=902
  _ROLLUPFUNCTION._serialized_start=904
  _ROLLUPFUNCTION._serialized_end=1005
  _QUERYTIMESERIESMETRICSRESPONSE._serialized_start=1008
  _QUERYTIMESERIESMETRICSRESPONSE._serialized_end=1140
  _METRICSERIES._serialized_start=1143
  _METRICSERIES._serialized_end=1274
  _DATAPOINT._serialized_start=1276
  _DATAPOINT._serialized_end=1321
  _QUOTALIMITSREQUEST._serialized_start=1323
  _QUOTALIMITSREQUEST._serialized_end=1343
  _QUOTALIMITSRESPONSE._serialized_start=1345
  _QUOTALIMITSRESPONSE._serialized_end=1426
  _QUOTAUSAGEREQUEST._serialized_start=1428
  _QUOTAUSAGEREQUEST._serialized_end=1447
  _QUOTAUSAGERESPONSE._serialized_start=1450
  _QUOTAUSAGERESPONSE._serialized_end=1617
  _GETINFOREQUEST._serialized_start=1619
  _GETINFOREQUEST._serialized_end=1635
  _GETINFORESPONSE._serialized_start=1637
  _GETINFORESPONSE._serialized_end=1729
  _OBSERVABILITY._serialized_start=2466
  _OBSERVABILITY._serialized_end=3341
# @@protoc_insertion_point(module_scope)



from google.protobuf import any_pb2 as _any_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class HttpBody(_message.Message):
    __slots__ = ["content_type", "data", "extensions"]
    CONTENT_TYPE_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    EXTENSIONS_FIELD_NUMBER: _ClassVar[int]
    content_type: str
    data: bytes
    extensions: _containers.RepeatedCompositeFieldContainer[_any_pb2.Any]
    def __init__(self, content_type: _Optional[str] = ..., data: _Optional[bytes] = ..., extensions: _Optional[_Iterable[_Union[_any_pb2.Any, _Mapping]]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: server/v1/api.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ...google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from ...openapiv3 import annotations_pb2 as openapiv3_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13server/v1/api.proto\x12\rtigrisdata.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1bopenapiv3/annotations.proto\"\x0e\n\x0cWriteOptions\"\x19\n\tCollation\x12\x0c\n\x04\x63\x61se\x18\x01 \x01(\t\"n\n\x12ReadRequestOptions\x12\r\n\x05limit\x18\x02 \x01(\x03\x12\x0c\n\x04skip\x18\x03 \x01(\x03\x12\x0e\n\x06offset\x18\x04 \x01(\x0c\x12+\n\tcollation\x18\x05 \x01(\x0b\x32\x18.tigrisdata.v1.Collation\"\x11\n\x0f\x44\x61tabaseOptions\"\x13\n\x11\x43ollectionOptions\"\x14\n\x12TransactionOptions\",\n\x0eTransactionCtx\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0e\n\x06origin\x18\x02 \x01(\t\"n\n\x17\x42\x65ginTransactionRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x32\n\x07options\x18\x02 \x01(\x0b\x32!.tigrisdata.v1.TransactionOptions\x12\x0e\n\x06\x62ranch\x18\x03 \x01(\t\"I\n\x18\x42\x65ginTransactionResponse\x12-\n\x06tx_ctx\x18\x01 \x01(\x0b\x32\x1d.tigrisdata.v1.TransactionCtx\";\n\x18\x43ommitTransactionRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x02 \x01(\t\"+\n\x19\x43ommitTransactionResponse\x12\x0e\n\x06status\x18\x01 \x01(\t\"=\n\x1aRollbackTransactionRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x02 \x01(\t\"-\n\x1bRollbackTransactionResponse\x12\x0e\n\x06status\x18\x01 \x01(\t\"J\n\x14InsertRequestOptions\x12\x32\n\rwrite_options\x18\x01 \x01(\x0b\x32\x1b.tigrisdata.v1.WriteOptions\"J\n\x14ImportRequestOptions\x12\x32\n\rwrite_options\x18\x01 \x01(\x0b\x32\x1b.tigrisdata.v1.WriteOptions\"\xa2\x01\n\x10ResponseMetadata\x12.\n\ncreated_at\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nupdated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\ndeleted_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x8d\x01\n\rInsertRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\x11\n\tdocuments\x18\x03 \x03(\x0c\x12\x34\n\x07options\x18\x04 \x01(\x0b\x32#.tigrisdata.v1.InsertRequestOptions\x12\x0e\n\x06\x62ranch\x18\x05 \x01(\t\"a\n\x0eInsertResponse\x12\x31\n\x08metadata\x18\x01 \x01(\x0b\x32\x1f.tigrisdata.v1.ResponseMetadata\x12\x0e\n\x06status\x18\x02 \x01(\t\x12\x0c\n\x04keys\x18\x03 \x03(\x0c\"\xd4\x01\n\rImportRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\x11\n\tdocuments\x18\x03 \x03(\x0c\x12\x34\n\x07options\x18\x04 \x01(\x0b\x32#.tigrisdata.v1.ImportRequestOptions\x12\x0e\n\x06\x62ranch\x18\x05 \x01(\t\x12\x19\n\x11\x63reate_collection\x18\x06 \x01(\x08\x12\x13\n\x0bprimary_key\x18\x07 \x03(\t\x12\x15\n\rautogenerated\x18\x08 \x03(\t\"a\n\x0eImportResponse\x12\x31\n\x08metadata\x18\x01 \x01(\x0b\x32\x1f.tigrisdata.v1.ResponseMetadata\x12\x0e\n\x06status\x18\x02 \x01(\t\x12\x0c\n\x04keys\x18\x03 \x03(\x0c\"K\n\x15ReplaceRequestOptions\x12\x32\n\rwrite_options\x18\x01 \x01(\x0b\x32\x1b.tigrisdata.v1.WriteOptions\"\x8f\x01\n\x0eReplaceRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\x11\n\tdocuments\x18\x03 \x03(\x0c\x12\x35\n\x07options\x18\x04 \x01(\x0b\x32$.tigrisdata.v1.ReplaceRequestOptions\x12\x0e\n\x06\x62ranch\x18\x05 \x01(\t\"b\n\x0fReplaceResponse\x12\x31\n\x08metadata\x18\x01 \x01(\x0b\x32\x1f.tigrisdata.v1.ResponseMetadata\x12\x0e\n\x06status\x18\x02 \x01(\t\x12\x0c\n\x04keys\x18\x03 \x03(\x0c\"\x86\x01\n\x14\x44\x65leteRequestOptions\x12\x32\n\rwrite_options\x18\x01 \x01(\x0b\x32\x1b.tigrisdata.v1.WriteOptions\x12+\n\tcollation\x18\x02 \x01(\x0b\x32\x18.tigrisdata.v1.Collation\x12\r\n\x05limit\x18\x03 \x01(\x03\"\x8a\x01\n\rDeleteRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\x0e\n\x06\x66ilter\x18\x03 \x01(\x0c\x12\x34\n\x07options\x18\x04 \x01(\x0b\x32#.tigrisdata.v1.DeleteRequestOptions\x12\x0e\n\x06\x62ranch\x18\x05 \x01(\t\"j\n\x0e\x44\x65leteResponse\x12\x31\n\x08metadata\x18\x01 \x01(\x0b\x32\x1f.tigrisdata.v1.ResponseMetadata\x12\x0e\n\x06status\x18\x02 \x01(\t\x12\x15\n\rdeleted_count\x18\x03 \x01(\x05\"\x86\x01\n\x14UpdateRequestOptions\x12\x32\n\rwrite_options\x18\x01 \x01(\x0b\x32\x1b.tigrisdata.v1.WriteOptions\x12+\n\tcollation\x18\x02 \x01(\x0b\x32\x18.tigrisdata.v1.Collation\x12\r\n\x05limit\x18\x03 \x01(\x03\"\x9a\x01\n\rUpdateRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\x0e\n\x06\x66ields\x18\x03 \x01(\x0c\x12\x0e\n\x06\x66ilter\x18\x04 \x01(\x0c\x12\x34\n\x07options\x18\x05 \x01(\x0b\x32#.tigrisdata.v1.UpdateRequestOptions\x12\x0e\n\x06\x62ranch\x18\x06 \x01(\t\"k\n\x0eUpdateResponse\x12\x31\n\x08metadata\x18\x01 \x01(\x0b\x32\x1f.tigrisdata.v1.ResponseMetadata\x12\x16\n\x0emodified_count\x18\x02 \x01(\x05\x12\x0e\n\x06status\x18\x03 \x01(\t\"\xa4\x01\n\x0bReadRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\x0e\n\x06\x66ilter\x18\x03 \x01(\x0c\x12\x0e\n\x06\x66ields\x18\x04 \x01(\x0c\x12\x32\n\x07options\x18\x05 \x01(\x0b\x32!.tigrisdata.v1.ReadRequestOptions\x12\x0c\n\x04sort\x18\x06 \x01(\x0c\x12\x0e\n\x06\x62ranch\x18\x07 \x01(\t\"e\n\x0cReadResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\x14\n\x0cresume_token\x18\x02 \x01(\x0c\x12\x31\n\x08metadata\x18\x03 \x01(\x0b\x32\x1f.tigrisdata.v1.ResponseMetadata\"S\n\x0c\x43ountRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\x0e\n\x06\x66ilter\x18\x03 \x01(\x0c\x12\x0e\n\x06\x62ranch\x18\x04 \x01(\t\"\x1e\n\rCountResponse\x12\r\n\x05\x63ount\x18\x01 \x01(\x03\"{\n\x0f\x45xplainResponse\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\x11\n\tread_type\x18\x02 \x01(\t\x12\x0e\n\x06\x66ilter\x18\x03 \x01(\t\x12\x0f\n\x07sorting\x18\x04 \x01(\t\x12\x11\n\tkey_range\x18\x05 \x03(\t\x12\r\n\x05\x66ield\x18\x06 \x01(\t\"\xb3\x02\n\rSearchRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\t\n\x01q\x18\x03 \x01(\t\x12\x15\n\rsearch_fields\x18\x04 \x03(\t\x12\x0e\n\x06\x66ilter\x18\x05 \x01(\x0c\x12\r\n\x05\x66\x61\x63\x65t\x18\x06 \x01(\x0c\x12\x0c\n\x04sort\x18\x07 \x01(\x0c\x12\x16\n\x0einclude_fields\x18\x08 \x03(\t\x12\x16\n\x0e\x65xclude_fields\x18\t \x03(\t\x12\x11\n\tpage_size\x18\n \x01(\x05\x12\x0c\n\x04page\x18\x0b \x01(\x05\x12+\n\tcollation\x18\x0c \x01(\x0b\x32\x18.tigrisdata.v1.Collation\x12\x0e\n\x06\x62ranch\x18\r \x01(\t\x12\x10\n\x08group_by\x18\x0e \x01(\x0c\x12\x0e\n\x06vector\x18\x0f \x01(\x0c\"\x9c\x02\n\x0eSearchResponse\x12&\n\x04hits\x18\x01 \x03(\x0b\x32\x18.tigrisdata.v1.SearchHit\x12\x39\n\x06\x66\x61\x63\x65ts\x18\x02 \x03(\x0b\x32).tigrisdata.v1.SearchResponse.FacetsEntry\x12+\n\x04meta\x18\x03 \x01(\x0b\x32\x1d.tigrisdata.v1.SearchMetadata\x12/\n\x05group\x18\x04 \x03(\x0b\x32 .tigrisdata.v1.GroupedSearchHits\x1aI\n\x0b\x46\x61\x63\x65tsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.tigrisdata.v1.SearchFacet:\x02\x38\x01\"I\n\tSearchHit\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12.\n\x08metadata\x18\x02 \x01(\x0b\x32\x1c.tigrisdata.v1.SearchHitMeta\"\x94\x01\n\rSearchHitMeta\x12.\n\ncreated_at\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nupdated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12#\n\x05match\x18\x03 \x01(\x0b\x32\x14.tigrisdata.v1.Match\"b\n\x0bSearchFacet\x12)\n\x06\x63ounts\x18\x01 \x03(\x0b\x32\x19.tigrisdata.v1.FacetCount\x12(\n\x05stats\x18\x02 \x01(\x0b\x32\x19.tigrisdata.v1.FacetStats\"*\n\nFacetCount\x12\r\n\x05\x63ount\x18\x01 \x01(\x03\x12\r\n\x05value\x18\x02 \x01(\t\"\x83\x01\n\nFacetStats\x12\x10\n\x03\x61vg\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12\x10\n\x03max\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x10\n\x03min\x18\x03 \x01(\x01H\x02\x88\x01\x01\x12\x10\n\x03sum\x18\x04 \x01(\x01H\x03\x88\x01\x01\x12\r\n\x05\x63ount\x18\x05 \x01(\x03\x42\x06\n\x04_avgB\x06\n\x04_maxB\x06\n\x04_minB\x06\n\x04_sum\"o\n\x0eSearchMetadata\x12\r\n\x05\x66ound\x18\x01 \x01(\x03\x12\x13\n\x0btotal_pages\x18\x02 \x01(\x05\x12!\n\x04page\x18\x03 \x01(\x0b\x32\x13.tigrisdata.v1.Page\x12\x16\n\x0ematched_fields\x18\x04 \x03(\t\"\x1a\n\nMatchField\x12\x0c\n\x04name\x18\x01 \x01(\t\"s\n\x05Match\x12)\n\x06\x66ields\x18\x01 \x03(\x0b\x32\x19.tigrisdata.v1.MatchField\x12\r\n\x05score\x18\x02 \x01(\t\x12\x1c\n\x0fvector_distance\x18\x03 \x01(\x01H\x00\x88\x01\x01\x42\x12\n\x10_vector_distance\"O\n\x11GroupedSearchHits\x12\x12\n\ngroup_keys\x18\x01 \x03(\t\x12&\n\x04hits\x18\x02 \x03(\x0b\x32\x18.tigrisdata.v1.SearchHit\"%\n\x04Page\x12\x0f\n\x07\x63urrent\x18\x01 \x01(\x05\x12\x0c\n\x04size\x18\x03 \x01(\x05\"\'\n\x14\x43reateProjectRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\"8\n\x15\x43reateProjectResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\"\'\n\x14\x44\x65leteProjectRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\"8\n\x15\x44\x65leteProjectResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\"6\n\x13\x43reateBranchRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x02 \x01(\t\"7\n\x14\x43reateBranchResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\"6\n\x13\x44\x65leteBranchRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x02 \x01(\t\"7\n\x14\x44\x65leteBranchResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\"&\n\x13ListBranchesRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\"C\n\x14ListBranchesResponse\x12+\n\x08\x62ranches\x18\x01 \x03(\x0b\x32\x19.tigrisdata.v1.BranchInfo\"\xae\x01\n\x1f\x43reateOrUpdateCollectionRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\x0e\n\x06schema\x18\x03 \x01(\x0c\x12\x13\n\x0bonly_create\x18\x04 \x01(\x08\x12\x31\n\x07options\x18\x05 \x01(\x0b\x32 .tigrisdata.v1.CollectionOptions\x12\x0e\n\x06\x62ranch\x18\x06 \x01(\t\"C\n CreateOrUpdateCollectionResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\"\x7f\n\x15\x44ropCollectionRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\x31\n\x07options\x18\x03 \x01(\x0b\x32 .tigrisdata.v1.CollectionOptions\x12\x0e\n\x06\x62ranch\x18\x04 \x01(\t\"9\n\x16\x44ropCollectionResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\"P\n\x0bProjectInfo\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x30\n\x08metadata\x18\x02 \x01(\x0b\x32\x1e.tigrisdata.v1.ProjectMetadata\"Y\n\x0e\x43ollectionInfo\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\x33\n\x08metadata\x18\x02 \x01(\x0b\x32!.tigrisdata.v1.CollectionMetadata\"M\n\nBranchInfo\x12\x0e\n\x06\x62ranch\x18\x01 \x01(\t\x12/\n\x08metadata\x18\x02 \x01(\x0b\x32\x1d.tigrisdata.v1.BranchMetadata\"\x15\n\x13ListProjectsRequest\"D\n\x14ListProjectsResponse\x12,\n\x08projects\x18\x01 \x03(\x0b\x32\x1a.tigrisdata.v1.ProjectInfo\"l\n\x16ListCollectionsRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x31\n\x07options\x18\x02 \x01(\x0b\x32 .tigrisdata.v1.CollectionOptions\x12\x0e\n\x06\x62ranch\x18\x03 \x01(\t\"M\n\x17ListCollectionsResponse\x12\x32\n\x0b\x63ollections\x18\x02 \x03(\x0b\x32\x1d.tigrisdata.v1.CollectionInfo\"Q\n\x17\x44\x65scribeDatabaseRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x15\n\rschema_format\x18\x02 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x03 \x01(\t\"\x9a\x01\n\x19\x44\x65scribeCollectionRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\x15\n\rschema_format\x18\x03 \x01(\t\x12\x31\n\x07options\x18\x04 \x01(\x0b\x32 .tigrisdata.v1.CollectionOptions\x12\x0e\n\x06\x62ranch\x18\x05 \x01(\t\"\xa8\x01\n\x18\x44\x65scribeDatabaseResponse\x12\x31\n\x08metadata\x18\x01 \x01(\x0b\x32\x1f.tigrisdata.v1.DatabaseMetadata\x12\x39\n\x0b\x63ollections\x18\x02 \x03(\x0b\x32$.tigrisdata.v1.CollectionDescription\x12\x0c\n\x04size\x18\x03 \x01(\x03\x12\x10\n\x08\x62ranches\x18\x04 \x03(\t\"\xb4\x01\n\x1a\x44\x65scribeCollectionResponse\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\x33\n\x08metadata\x18\x02 \x01(\x0b\x32!.tigrisdata.v1.CollectionMetadata\x12\x0e\n\x06schema\x18\x03 \x01(\x0c\x12\x0c\n\x04size\x18\x04 \x01(\x03\x12/\n\x07indexes\x18\x05 \x03(\x0b\x32\x1e.tigrisdata.v1.CollectionIndex\"\x14\n\x12ProjectDescription\"\xaf\x01\n\x15\x43ollectionDescription\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\x33\n\x08metadata\x18\x02 \x01(\x0b\x32!.tigrisdata.v1.CollectionMetadata\x12\x0e\n\x06schema\x18\x03 \x01(\x0c\x12\x0c\n\x04size\x18\x04 \x01(\x03\x12/\n\x07indexes\x18\x05 \x03(\x0b\x32\x1e.tigrisdata.v1.CollectionIndex\"\x11\n\x0fProjectMetadata\"\x12\n\x10\x44\x61tabaseMetadata\"\x14\n\x12\x43ollectionMetadata\"T\n\x0f\x43ollectionIndex\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05state\x18\x02 \x01(\t\x12$\n\x06\x66ields\x18\x03 \x03(\x0b\x32\x14.tigrisdata.v1.Field\"\x15\n\x05\x46ield\x12\x0c\n\x04name\x18\x01 \x01(\t\"R\n\x1b\x42uildCollectionIndexRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x12\n\ncollection\x18\x02 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x03 \x01(\t\"O\n\x1c\x42uildCollectionIndexResponse\x12/\n\x07indexes\x18\x01 \x03(\x0b\x32\x1e.tigrisdata.v1.CollectionIndex\"\x10\n\x0e\x42ranchMetadata\"I\n\x13\x43reateAppKeyRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0f\n\x07project\x18\x03 \x01(\t\"F\n\x14\x43reateAppKeyResponse\x12.\n\x0f\x63reated_app_key\x18\x01 \x01(\x0b\x32\x15.tigrisdata.v1.AppKey\"U\n\x13UpdateAppKeyRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\"F\n\x14UpdateAppKeyResponse\x12.\n\x0fupdated_app_key\x18\x01 \x01(\x0b\x32\x15.tigrisdata.v1.AppKey\"\xa8\x01\n\x06\x41ppKey\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x0e\n\x06secret\x18\x04 \x01(\t\x12\x12\n\ncreated_at\x18\x05 \x01(\x03\x12\x12\n\ncreated_by\x18\x06 \x01(\t\x12\x12\n\nupdated_at\x18\x07 \x01(\x03\x12\x12\n\nupdated_by\x18\x08 \x01(\t\x12\x0f\n\x07project\x18\t \x01(\t\"%\n\x12ListAppKeysRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\">\n\x13ListAppKeysResponse\x12\'\n\x08\x61pp_keys\x18\x01 \x03(\x0b\x32\x15.tigrisdata.v1.AppKey\"2\n\x13\x44\x65leteAppKeyRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\"\'\n\x14\x44\x65leteAppKeyResponse\x12\x0f\n\x07\x64\x65leted\x18\x01 \x01(\x08\"2\n\x13RotateAppKeyRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\">\n\x14RotateAppKeyResponse\x12&\n\x07\x61pp_key\x18\x01 \x01(\x0b\x32\x15.tigrisdata.v1.AppKey2\xba,\n\x06Tigris\x12\xc4\x01\n\x10\x42\x65ginTransaction\x12&.tigrisdata.v1.BeginTransactionRequest\x1a\'.tigrisdata.v1.BeginTransactionResponse\"_\x82\xd3\xe4\x93\x02\x37\"2/v1/projects/{project}/database/transactions/begin:\x01*\xbaG\x1f\n\x08\x44\x61tabase\x12\x13\x42\x65gin a transaction\x12\xc9\x01\n\x11\x43ommitTransaction\x12\'.tigrisdata.v1.CommitTransactionRequest\x1a(.tigrisdata.v1.CommitTransactionResponse\"a\x82\xd3\xe4\x93\x02\x38\"3/v1/projects/{project}/database/transactions/commit:\x01*\xbaG \n\x08\x44\x61tabase\x12\x14\x43ommit a Transaction\x12\xd3\x01\n\x13RollbackTransaction\x12).tigrisdata.v1.RollbackTransactionRequest\x1a*.tigrisdata.v1.RollbackTransactionResponse\"e\x82\xd3\xe4\x93\x02:\"5/v1/projects/{project}/database/transactions/rollback:\x01*\xbaG\"\n\x08\x44\x61tabase\x12\x16Rollback a transaction\x12\xbd\x01\n\x06Insert\x12\x1c.tigrisdata.v1.InsertRequest\x1a\x1d.tigrisdata.v1.InsertResponse\"v\x82\xd3\xe4\x93\x02N\"I/v1/projects/{project}/database/collections/{collection}/documents/insert:\x01*\xbaG\x1f\n\x0b\x43ollections\x12\x10Insert Documents\x12\xcd\x01\n\x07Replace\x12\x1d.tigrisdata.v1.ReplaceRequest\x1a\x1e.tigrisdata.v1.ReplaceResponse\"\x82\x01\x82\xd3\xe4\x93\x02O\x1aJ/v1/projects/{project}/database/collections/{collection}/documents/replace:\x01*\xbaG*\n\x0b\x43ollections\x12\x1bInsert or Replace Documents\x12\xbd\x01\n\x06\x44\x65lete\x12\x1c.tigrisdata.v1.DeleteRequest\x1a\x1d.tigrisdata.v1.DeleteResponse\"v\x82\xd3\xe4\x93\x02N*I/v1/projects/{project}/database/collections/{collection}/documents/delete:\x01*\xbaG\x1f\n\x0b\x43ollections\x12\x10\x44\x65lete Documents\x12\xbe\x01\n\x06Update\x12\x1c.tigrisdata.v1.UpdateRequest\x1a\x1d.tigrisdata.v1.UpdateResponse\"w\x82\xd3\xe4\x93\x02N\x1aI/v1/projects/{project}/database/collections/{collection}/documents/update:\x01*\xbaG \n\x0b\x43ollections\x12\x11Update Documents.\x12\xb5\x01\n\x04Read\x12\x1a.tigrisdata.v1.ReadRequest\x1a\x1b.tigrisdata.v1.ReadResponse\"r\x82\xd3\xe4\x93\x02L\"G/v1/projects/{project}/database/collections/{collection}/documents/read:\x01*\xbaG\x1d\n\x0b\x43ollections\x12\x0eRead Documents0\x01\x12\xb8\x01\n\x05\x43ount\x12\x1b.tigrisdata.v1.CountRequest\x1a\x1c.tigrisdata.v1.CountResponse\"t\x82\xd3\xe4\x93\x02M\"H/v1/projects/{project}/database/collections/{collection}/documents/count:\x01*\xbaG\x1e\n\x0b\x43ollections\x12\x0f\x43ount Documents\x12\xfc\x01\n\x14\x42uildCollectionIndex\x12*.tigrisdata.v1.BuildCollectionIndexRequest\x1a+.tigrisdata.v1.BuildCollectionIndexResponse\"\x8a\x01\x82\xd3\xe4\x93\x02M\"H/v1/projects/{project}/database/collections/{collection}/documents/index:\x01*\xbaG4\n\x0b\x43ollections\x12%Index all documents in the collection\x12\xc4\x01\n\x07\x45xplain\x12\x1a.tigrisdata.v1.ReadRequest\x1a\x1e.tigrisdata.v1.ExplainResponse\"}\x82\xd3\xe4\x93\x02O\"J/v1/projects/{project}/database/collections/{collection}/documents/explain:\x01*\xbaG%\n\x0b\x43ollections\x12\x16\x45xplain Read Documents\x12\xc0\x01\n\x06Search\x12\x1c.tigrisdata.v1.SearchRequest\x1a\x1d.tigrisdata.v1.SearchResponse\"w\x82\xd3\xe4\x93\x02N\"I/v1/projects/{project}/database/collections/{collection}/documents/search:\x01*\xbaG \n\x0b\x43ollections\x12\x11Search Documents.0\x01\x12\xbd\x01\n\x06Import\x12\x1c.tigrisdata.v1.ImportRequest\x1a\x1d.tigrisdata.v1.ImportResponse\"v\x82\xd3\xe4\x93\x02N\"I/v1/projects/{project}/database/collections/{collection}/documents/import:\x01*\xbaG\x1f\n\x0b\x43ollections\x12\x10Import Documents\x12\xff\x01\n\x18\x43reateOrUpdateCollection\x12..tigrisdata.v1.CreateOrUpdateCollectionRequest\x1a/.tigrisdata.v1.CreateOrUpdateCollectionResponse\"\x81\x01\x82\xd3\xe4\x93\x02L\"G/v1/projects/{project}/database/collections/{collection}/createOrUpdate:\x01*\xbaG,\n\x0b\x43ollections\x12\x1d\x43reate or update a collection\x12\xc8\x01\n\x0e\x44ropCollection\x12$.tigrisdata.v1.DropCollectionRequest\x1a%.tigrisdata.v1.DropCollectionResponse\"i\x82\xd3\xe4\x93\x02\x42*=/v1/projects/{project}/database/collections/{collection}/drop:\x01*\xbaG\x1e\n\x0b\x43ollections\x12\x0f\x44rop Collection\x12\x89\x01\n\x0cListProjects\x12\".tigrisdata.v1.ListProjectsRequest\x1a#.tigrisdata.v1.ListProjectsResponse\"0\x82\xd3\xe4\x93\x02\x0e\x12\x0c/v1/projects\xbaG\x19\n\x08Projects\x12\rList Projects\x12\xb4\x01\n\x0fListCollections\x12%.tigrisdata.v1.ListCollectionsRequest\x1a&.tigrisdata.v1.ListCollectionsResponse\"R\x82\xd3\xe4\x93\x02-\x12+/v1/projects/{project}/database/collections\xbaG\x1c\n\x08\x44\x61tabase\x12\x10List Collections\x12\xa1\x01\n\rCreateProject\x12#.tigrisdata.v1.CreateProjectRequest\x1a$.tigrisdata.v1.CreateProjectResponse\"E\x82\xd3\xe4\x93\x02\"\"\x1d/v1/projects/{project}/create:\x01*\xbaG\x1a\n\x08Projects\x12\x0e\x43reate Project\x12\xc1\x01\n\rDeleteProject\x12#.tigrisdata.v1.DeleteProjectRequest\x1a$.tigrisdata.v1.DeleteProjectResponse\"e\x82\xd3\xe4\x93\x02\"*\x1d/v1/projects/{project}/delete:\x01*\xbaG:\n\x08Projects\x12.Delete Project and all resources under project\x12\xb8\x01\n\x10\x44\x65scribeDatabase\x12&.tigrisdata.v1.DescribeDatabaseRequest\x1a\'.tigrisdata.v1.DescribeDatabaseResponse\"S\x82\xd3\xe4\x93\x02-\"(/v1/projects/{project}/database/describe:\x01*\xbaG\x1d\n\x08\x44\x61tabase\x12\x11\x44\x65scribe database\x12\xc3\x01\n\x0c\x43reateBranch\x12\".tigrisdata.v1.CreateBranchRequest\x1a#.tigrisdata.v1.CreateBranchResponse\"j\x82\xd3\xe4\x93\x02=\"8/v1/projects/{project}/database/branches/{branch}/create:\x01*\xbaG$\n\x08\x44\x61tabase\x12\x18\x43reate a database branch\x12\xc3\x01\n\x0c\x44\x65leteBranch\x12\".tigrisdata.v1.DeleteBranchRequest\x1a#.tigrisdata.v1.DeleteBranchResponse\"j\x82\xd3\xe4\x93\x02=*8/v1/projects/{project}/database/branches/{branch}/delete:\x01*\xbaG$\n\x08\x44\x61tabase\x12\x18\x44\x65lete a database branch\x12\xae\x01\n\x0cListBranches\x12\".tigrisdata.v1.ListBranchesRequest\x1a#.tigrisdata.v1.ListBranchesResponse\"U\x82\xd3\xe4\x93\x02*\x12(/v1/projects/{project}/database/branches\xbaG\"\n\x08\x44\x61tabase\x12\x16List database branches\x12\xdc\x01\n\x12\x44\x65scribeCollection\x12(.tigrisdata.v1.DescribeCollectionRequest\x1a).tigrisdata.v1.DescribeCollectionResponse\"q\x82\xd3\xe4\x93\x02\x46\"A/v1/projects/{project}/database/collections/{collection}/describe:\x01*\xbaG\"\n\x0b\x43ollections\x12\x13\x44\x65scribe Collection\x12\xb5\x01\n\x0c\x43reateAppKey\x12\".tigrisdata.v1.CreateAppKeyRequest\x1a#.tigrisdata.v1.CreateAppKeyResponse\"\\\x82\xd3\xe4\x93\x02,\"\'/v1/projects/{project}/apps/keys/create:\x01*\xbaG\'\n\x10\x41pplication Keys\x12\x13\x43reates the app key\x12\xc8\x01\n\x0cUpdateAppKey\x12\".tigrisdata.v1.UpdateAppKeyRequest\x1a#.tigrisdata.v1.UpdateAppKeyResponse\"o\x82\xd3\xe4\x93\x02,\"\'/v1/projects/{project}/apps/keys/update:\x01*\xbaG:\n\x10\x41pplication Keys\x12&Updates the description of the app key\x12\xb5\x01\n\x0c\x44\x65leteAppKey\x12\".tigrisdata.v1.DeleteAppKeyRequest\x1a#.tigrisdata.v1.DeleteAppKeyResponse\"\\\x82\xd3\xe4\x93\x02,*\'/v1/projects/{project}/apps/keys/delete:\x01*\xbaG\'\n\x10\x41pplication Keys\x12\x13\x44\x65letes the app key\x12\xaa\x01\n\x0bListAppKeys\x12!.tigrisdata.v1.ListAppKeysRequest\x1a\".tigrisdata.v1.ListAppKeysResponse\"T\x82\xd3\xe4\x93\x02\"\x12 /v1/projects/{project}/apps/keys\xbaG)\n\x10\x41pplication Keys\x12\x15List all the app keys\x12\xc2\x01\n\x12RotateAppKeySecret\x12\".tigrisdata.v1.RotateAppKeyRequest\x1a#.tigrisdata.v1.RotateAppKeyResponse\"c\x82\xd3\xe4\x93\x02,\"\'/v1/projects/{project}/apps/keys/rotate:\x01*\xbaG.\n\x10\x41pplication Keys\x12\x1aRotates the app key secretB\x80\x13\n\x1d\x63om.tigrisdata.db.api.v1.grpcZ github.com/tigrisdata/tigris/api\xbaG\xbb\x12\x12\x96\x02\n\x14Tigris API Reference\x12\xac\x01\n## Overview\nThis section is organized around HTTP APIs. The APIs accepts JSON requests and returns JSON-encoded responses. The APIs conforms to standard HTTP status codes.*H\n\x0e\x41pache License\x12\x36https://github.com/tigrisdata/tigris/blob/main/LICENSE2\x05\x30.0.1\x1a\x34\n$https://api.preview.tigrisdata.cloud\x12\x0cTigris Cloud\x1a\"\n\x15http://localhost:8081\x12\tLocalhost*\": \n\x1e\n\nBearerAuth\x12\x10\n\x0e\n\x04http*\x06\x62\x65\x61rer2\x10\n\x0e\n\nBearerAuth\x12\x00:\x99\x03\n\x0e\x41uthentication\x12\x86\x03The auth section of API provides OAuth 2.0 APIs. Tigris supports pluggable OAuth provider. Pass the token in the headers for authentication, as an example `-H \"Authorization: Bearer eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCIsImtpZCI6I\"`(replace it with your token). All API requests must be made over HTTPS. Calls made over plain HTTP will fail. API requests without authentication will also fail.:\xb4\x01\n\x10\x41pplication Keys\x12\x9f\x01The application keys section provide APIs that can be used to manage application keys for your project. A single project can have one or more application keys.:\x82\x02\n\x08Projects\x12\xf5\x01\x45very Tigris projects comes with a transactional document database built on FoundationDB, one of the most resilient and battle-tested open source distributed key-value store. A database is created automatically for you when you create a project.:\xd9\x01\n\x08\x44\x61tabase\x12\xcc\x01The Database section provide APIs that can be used to interact with the database. A single Database can have one or more collections. A database is created automatically for you when you create a project.:\x88\x01\n\x0b\x43ollections\x12yThe Collections section provide APIs that can be used to manage collections. A collection can have one or more documents.:a\n\nManagement\x12SThe Management section provide APIs that can be used to manage users, and app_keys.:\x89\x01\n\rObservability\x12xThe Observability section has APIs that provides full visibility into the health, metrics, and monitoring of the Server.:U\n\x05\x43\x61\x63he\x12LThe cache section provide APIs that can be used to perform cache operations.:S\n\x08Realtime\x12GThe realtime section provide APIs that can be used realtime operations.:\xd4\x01\n\x06Search\x12\xc9\x01The search section provides you APIs that can be used to implement powerful apps with search experiences. You can manage storing documents on your own or you can simply integrate it with your database.:_\n\x07\x42illing\x12TThe billing section provides APIs for managing invoices and payments infrastructure.b\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'server.v1.api_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\035com.tigrisdata.db.api.v1.grpcZ github.com/tigrisdata/tigris/api\272G\273\022\022\226\002\n\024Tigris API Reference\022\254\001\n## Overview\nThis section is organized around HTTP APIs. The APIs accepts JSON requests and returns JSON-encoded responses. The APIs conforms to standard HTTP status codes.*H\n\016Apache License\0226https://github.com/tigrisdata/tigris/blob/main/LICENSE2\0050.0.1\0324\n$https://api.preview.tigrisdata.cloud\022\014Tigris Cloud\032\"\n\025http://localhost:8081\022\tLocalhost*\": \n\036\n\nBearerAuth\022\020\n\016\n\004http*\006bearer2\020\n\016\n\nBearerAuth\022\000:\231\003\n\016Authentication\022\206\003The auth section of API provides OAuth 2.0 APIs. Tigris supports pluggable OAuth provider. Pass the token in the headers for authentication, as an example `-H \"Authorization: Bearer eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCIsImtpZCI6I\"`(replace it with your token). All API requests must be made over HTTPS. Calls made over plain HTTP will fail. API requests without authentication will also fail.:\264\001\n\020Application Keys\022\237\001The application keys section provide APIs that can be used to manage application keys for your project. A single project can have one or more application keys.:\202\002\n\010Projects\022\365\001Every Tigris projects comes with a transactional document database built on FoundationDB, one of the most resilient and battle-tested open source distributed key-value store. A database is created automatically for you when you create a project.:\331\001\n\010Database\022\314\001The Database section provide APIs that can be used to interact with the database. A single Database can have one or more collections. A database is created automatically for you when you create a project.:\210\001\n\013Collections\022yThe Collections section provide APIs that can be used to manage collections. A collection can have one or more documents.:a\n\nManagement\022SThe Management section provide APIs that can be used to manage users, and app_keys.:\211\001\n\rObservability\022xThe Observability section has APIs that provides full visibility into the health, metrics, and monitoring of the Server.:U\n\005Cache\022LThe cache section provide APIs that can be used to perform cache operations.:S\n\010Realtime\022GThe realtime section provide APIs that can be used realtime operations.:\324\001\n\006Search\022\311\001The search section provides you APIs that can be used to implement powerful apps with search experiences. You can manage storing documents on your own or you can simply integrate it with your database.:_\n\007Billing\022TThe billing section provides APIs for managing invoices and payments infrastructure.'
  _SEARCHRESPONSE_FACETSENTRY._options = None
  _SEARCHRESPONSE_FACETSENTRY._serialized_options = b'8\001'
  _TIGRIS.methods_by_name['BeginTransaction']._options = None
  _TIGRIS.methods_by_name['BeginTransaction']._serialized_options = b'\202\323\344\223\0027\"2/v1/projects/{project}/database/transactions/begin:\001*\272G\037\n\010Database\022\023Begin a transaction'
  _TIGRIS.methods_by_name['CommitTransaction']._options = None
  _TIGRIS.methods_by_name['CommitTransaction']._serialized_options = b'\202\323\344\223\0028\"3/v1/projects/{project}/database/transactions/commit:\001*\272G \n\010Database\022\024Commit a Transaction'
  _TIGRIS.methods_by_name['RollbackTransaction']._options = None
  _TIGRIS.methods_by_name['RollbackTransaction']._serialized_options = b'\202\323\344\223\002:\"5/v1/projects/{project}/database/transactions/rollback:\001*\272G\"\n\010Database\022\026Rollback a transaction'
  _TIGRIS.methods_by_name['Insert']._options = None
  _TIGRIS.methods_by_name['Insert']._serialized_options = b'\202\323\344\223\002N\"I/v1/projects/{project}/database/collections/{collection}/documents/insert:\001*\272G\037\n\013Collections\022\020Insert Documents'
  _TIGRIS.methods_by_name['Replace']._options = None
  _TIGRIS.methods_by_name['Replace']._serialized_options = b'\202\323\344\223\002O\032J/v1/projects/{project}/database/collections/{collection}/documents/replace:\001*\272G*\n\013Collections\022\033Insert or Replace Documents'
  _TIGRIS.methods_by_name['Delete']._options = None
  _TIGRIS.methods_by_name['Delete']._serialized_options = b'\202\323\344\223\002N*I/v1/projects/{project}/database/collections/{collection}/documents/delete:\001*\272G\037\n\013Collections\022\020Delete Documents'
  _TIGRIS.methods_by_name['Update']._options = None
  _TIGRIS.methods_by_name['Update']._serialized_options = b'\202\323\344\223\002N\032I/v1/projects/{project}/database/collections/{collection}/documents/update:\001*\272G \n\013Collections\022\021Update Documents.'
  _TIGRIS.methods_by_name['Read']._options = None
  _TIGRIS.methods_by_name['Read']._serialized_options = b'\202\323\344\223\002L\"G/v1/projects/{project}/database/collections/{collection}/documents/read:\001*\272G\035\n\013Collections\022\016Read Documents'
  _TIGRIS.methods_by_name['Count']._options = None
  _TIGRIS.methods_by_name['Count']._serialized_options = b'\202\323\344\223\002M\"H/v1/projects/{project}/database/collections/{collection}/documents/count:\001*\272G\036\n\013Collections\022\017Count Documents'
  _TIGRIS.methods_by_name['BuildCollectionIndex']._options = None
  _TIGRIS.methods_by_name['BuildCollectionIndex']._serialized_options = b'\202\323\344\223\002M\"H/v1/projects/{project}/database/collections/{collection}/documents/index:\001*\272G4\n\013Collections\022%Index all documents in the collection'
  _TIGRIS.methods_by_name['Explain']._options = None
  _TIGRIS.methods_by_name['Explain']._serialized_options = b'\202\323\344\223\002O\"J/v1/projects/{project}/database/collections/{collection}/documents/explain:\001*\272G%\n\013Collections\022\026Explain Read Documents'
  _TIGRIS.methods_by_name['Search']._options = None
  _TIGRIS.methods_by_name['Search']._serialized_options = b'\202\323\344\223\002N\"I/v1/projects/{project}/database/collections/{collection}/documents/search:\001*\272G \n\013Collections\022\021Search Documents.'
  _TIGRIS.methods_by_name['Import']._options = None
  _TIGRIS.methods_by_name['Import']._serialized_options = b'\202\323\344\223\002N\"I/v1/projects/{project}/database/collections/{collection}/documents/import:\001*\272G\037\n\013Collections\022\020Import Documents'
  _TIGRIS.methods_by_name['CreateOrUpdateCollection']._options = None
  _TIGRIS.methods_by_name['CreateOrUpdateCollection']._serialized_options = b'\202\323\344\223\002L\"G/v1/projects/{project}/database/collections/{collection}/createOrUpdate:\001*\272G,\n\013Collections\022\035Create or update a collection'
  _TIGRIS.methods_by_name['DropCollection']._options = None
  _TIGRIS.methods_by_name['DropCollection']._serialized_options = b'\202\323\344\223\002B*=/v1/projects/{project}/database/collections/{collection}/drop:\001*\272G\036\n\013Collections\022\017Drop Collection'
  _TIGRIS.methods_by_name['ListProjects']._options = None
  _TIGRIS.methods_by_name['ListProjects']._serialized_options = b'\202\323\344\223\002\016\022\014/v1/projects\272G\031\n\010Projects\022\rList Projects'
  _TIGRIS.methods_by_name['ListCollections']._options = None
  _TIGRIS.methods_by_name['ListCollections']._serialized_options = b'\202\323\344\223\002-\022+/v1/projects/{project}/database/collections\272G\034\n\010Database\022\020List Collections'
  _TIGRIS.methods_by_name['CreateProject']._options = None
  _TIGRIS.methods_by_name['CreateProject']._serialized_options = b'\202\323\344\223\002\"\"\035/v1/projects/{project}/create:\001*\272G\032\n\010Projects\022\016Create Project'
  _TIGRIS.methods_by_name['DeleteProject']._options = None
  _TIGRIS.methods_by_name['DeleteProject']._serialized_options = b'\202\323\344\223\002\"*\035/v1/projects/{project}/delete:\001*\272G:\n\010Projects\022.Delete Project and all resources under project'
  _TIGRIS.methods_by_name['DescribeDatabase']._options = None
  _TIGRIS.methods_by_name['DescribeDatabase']._serialized_options = b'\202\323\344\223\002-\"(/v1/projects/{project}/database/describe:\001*\272G\035\n\010Database\022\021Describe database'
  _TIGRIS.methods_by_name['CreateBranch']._options = None
  _TIGRIS.methods_by_name['CreateBranch']._serialized_options = b'\202\323\344\223\002=\"8/v1/projects/{project}/database/branches/{branch}/create:\001*\272G$\n\010Database\022\030Create a database branch'
  _TIGRIS.methods_by_name['DeleteBranch']._options = None
  _TIGRIS.methods_by_name['DeleteBranch']._serialized_options = b'\202\323\344\223\002=*8/v1/projects/{project}/database/branches/{branch}/delete:\001*\272G$\n\010Database\022\030Delete a database branch'
  _TIGRIS.methods_by_name['ListBranches']._options = None
  _TIGRIS.methods_by_name['ListBranches']._serialized_options = b'\202\323\344\223\002*\022(/v1/projects/{project}/database/branches\272G\"\n\010Database\022\026List database branches'
  _TIGRIS.methods_by_name['DescribeCollection']._options = None
  _TIGRIS.methods_by_name['DescribeCollection']._serialized_options = b'\202\323\344\223\002F\"A/v1/projects/{project}/database/collections/{collection}/describe:\001*\272G\"\n\013Collections\022\023Describe Collection'
  _TIGRIS.methods_by_name['CreateAppKey']._options = None
  _TIGRIS.methods_by_name['CreateAppKey']._serialized_options = b'\202\323\344\223\002,\"\'/v1/projects/{project}/apps/keys/create:\001*\272G\'\n\020Application Keys\022\023Creates the app key'
  _TIGRIS.methods_by_name['UpdateAppKey']._options = None
  _TIGRIS.methods_by_name['UpdateAppKey']._serialized_options = b'\202\323\344\223\002,\"\'/v1/projects/{project}/apps/keys/update:\001*\272G:\n\020Application Keys\022&Updates the description of the app key'
  _TIGRIS.methods_by_name['DeleteAppKey']._options = None
  _TIGRIS.methods_by_name['DeleteAppKey']._serialized_options = b'\202\323\344\223\002,*\'/v1/projects/{project}/apps/keys/delete:\001*\272G\'\n\020Application Keys\022\023Deletes the app key'
  _TIGRIS.methods_by_name['ListAppKeys']._options = None
  _TIGRIS.methods_by_name['ListAppKeys']._serialized_options = b'\202\323\344\223\002\"\022 /v1/projects/{project}/apps/keys\272G)\n\020Application Keys\022\025List all the app keys'
  _TIGRIS.methods_by_name['RotateAppKeySecret']._options = None
  _TIGRIS.methods_by_name['RotateAppKeySecret']._serialized_options = b'\202\323\344\223\002,\"\'/v1/projects/{project}/apps/keys/rotate:\001*\272G.\n\020Application Keys\022\032Rotates the app key secret'
  _WRITEOPTIONS._serialized_start=130
  _WRITEOPTIONS._serialized_end=144
  _COLLATION._serialized_start=146
  _COLLATION._serialized_end=171
  _READREQUESTOPTIONS._serialized_start=173
  _READREQUESTOPTIONS._serialized_end=283
  _DATABASEOPTIONS._serialized_start=285
  _DATABASEOPTIONS._serialized_end=302
  _COLLECTIONOPTIONS._serialized_start=304
  _COLLECTIONOPTIONS._serialized_end=323
  _TRANSACTIONOPTIONS._serialized_start=325
  _TRANSACTIONOPTIONS._serialized_end=345
  _TRANSACTIONCTX._serialized_start=347
  _TRANSACTIONCTX._serialized_end=391
  _BEGINTRANSACTIONREQUEST._serialized_start=393
  _BEGINTRANSACTIONREQUEST._serialized_end=503
  _BEGINTRANSACTIONRESPONSE._serialized_start=505
  _BEGINTRANSACTIONRESPONSE._serialized_end=578
  _COMMITTRANSACTIONREQUEST._serialized_start=580
  _COMMITTRANSACTIONREQUEST._serialized_end=639
  _COMMITTRANSACTIONRESPONSE._serialized_start=641
  _COMMITTRANSACTIONRESPONSE._serialized_end=684
  _ROLLBACKTRANSACTIONREQUEST._serialized_start=686
  _ROLLBACKTRANSACTIONREQUEST._serialized_end=747
  _ROLLBACKTRANSACTIONRESPONSE._serialized_start=749
  _ROLLBACKTRANSACTIONRESPONSE._serialized_end=794
  _INSERTREQUESTOPTIONS._serialized_start=796
  _INSERTREQUESTOPTIONS._serialized_end=870
  _IMPORTREQUESTOPTIONS._serialized_start=872
  _IMPORTREQUESTOPTIONS._serialized_end=946
  _RESPONSEMETADATA._serialized_start=949
  _RESPONSEMETADATA._serialized_end=1111
  _INSERTREQUEST._serialized_start=1114
  _INSERTREQUEST._serialized_end=1255
  _INSERTRESPONSE._serialized_start=1257
  _INSERTRESPONSE._serialized_end=1354
  _IMPORTREQUEST._serialized_start=1357
  _IMPORTREQUEST._serialized_end=1569
  _IMPORTRESPONSE._serialized_start=1571
  _IMPORTRESPONSE._serialized_end=1668
  _REPLACEREQUESTOPTIONS._serialized_start=1670
  _REPLACEREQUESTOPTIONS._serialized_end=1745
  _REPLACEREQUEST._serialized_start=1748
  _REPLACEREQUEST._serialized_end=1891
  _REPLACERESPONSE._serialized_start=1893
  _REPLACERESPONSE._serialized_end=1991
  _DELETEREQUESTOPTIONS._serialized_start=1994
  _DELETEREQUESTOPTIONS._serialized_end=2128
  _DELETEREQUEST._serialized_start=2131
  _DELETEREQUEST._serialized_end=2269
  _DELETERESPONSE._serialized_start=2271
  _DELETERESPONSE._serialized_end=2377
  _UPDATEREQUESTOPTIONS._serialized_start=2380
  _UPDATEREQUESTOPTIONS._serialized_end=2514
  _UPDATEREQUEST._serialized_start=2517
  _UPDATEREQUEST._serialized_end=2671
  _UPDATERESPONSE._serialized_start=2673
  _UPDATERESPONSE._serialized_end=2780
  _READREQUEST._serialized_start=2783
  _READREQUEST._serialized_end=2947
  _READRESPONSE._serialized_start=2949
  _READRESPONSE._serialized_end=3050
  _COUNTREQUEST._serialized_start=3052
  _COUNTREQUEST._serialized_end=3135
  _COUNTRESPONSE._serialized_start=3137
  _COUNTRESPONSE._serialized_end=3167
  _EXPLAINRESPONSE._serialized_start=3169
  _EXPLAINRESPONSE._serialized_end=3292
  _SEARCHREQUEST._serialized_start=3295
  _SEARCHREQUEST._serialized_end=3602
  _SEARCHRESPONSE._serialized_start=3605
  _SEARCHRESPONSE._serialized_end=3889
  _SEARCHRESPONSE_FACETSENTRY._serialized_start=3816
  _SEARCHRESPONSE_FACETSENTRY._serialized_end=3889
  _SEARCHHIT._serialized_start=3891
  _SEARCHHIT._serialized_end=3964
  _SEARCHHITMETA._serialized_start=3967
  _SEARCHHITMETA._serialized_end=4115
  _SEARCHFACET._serialized_start=4117
  _SEARCHFACET._serialized_end=4215
  _FACETCOUNT._serialized_start=4217
  _FACETCOUNT._serialized_end=4259
  _FACETSTATS._serialized_start=4262
  _FACETSTATS._serialized_end=4393
  _SEARCHMETADATA._serialized_start=4395
  _SEARCHMETADATA._serialized_end=4506
  _MATCHFIELD._serialized_start=4508
  _MATCHFIELD._serialized_end=4534
  _MATCH._serialized_start=4536
  _MATCH._serialized_end=4651
  _GROUPEDSEARCHHITS._serialized_start=4653
  _GROUPEDSEARCHHITS._serialized_end=4732
  _PAGE._serialized_start=4734
  _PAGE._serialized_end=4771
  _CREATEPROJECTREQUEST._serialized_start=4773
  _CREATEPROJECTREQUEST._serialized_end=4812
  _CREATEPROJECTRESPONSE._serialized_start=4814
  _CREATEPROJECTRESPONSE._serialized_end=4870
  _DELETEPROJECTREQUEST._serialized_start=4872
  _DELETEPROJECTREQUEST._serialized_end=4911
  _DELETEPROJECTRESPONSE._serialized_start=4913
  _DELETEPROJECTRESPONSE._serialized_end=4969
  _CREATEBRANCHREQUEST._serialized_start=4971
  _CREATEBRANCHREQUEST._serialized_end=5025
  _CREATEBRANCHRESPONSE._serialized_start=5027
  _CREATEBRANCHRESPONSE._serialized_end=5082
  _DELETEBRANCHREQUEST._serialized_start=5084
  _DELETEBRANCHREQUEST._serialized_end=5138
  _DELETEBRANCHRESPONSE._serialized_start=5140
  _DELETEBRANCHRESPONSE._serialized_end=5195
  _LISTBRANCHESREQUEST._serialized_start=5197
  _LISTBRANCHESREQUEST._serialized_end=5235
  _LISTBRANCHESRESPONSE._serialized_start=5237
  _LISTBRANCHESRESPONSE._serialized_end=5304
  _CREATEORUPDATECOLLECTIONREQUEST._serialized_start=5307
  _CREATEORUPDATECOLLECTIONREQUEST._serialized_end=5481
  _CREATEORUPDATECOLLECTIONRESPONSE._serialized_start=5483
  _CREATEORUPDATECOLLECTIONRESPONSE._serialized_end=5550
  _DROPCOLLECTIONREQUEST._serialized_start=5552
  _DROPCOLLECTIONREQUEST._serialized_end=5679
  _DROPCOLLECTIONRESPONSE._serialized_start=5681
  _DROPCOLLECTIONRESPONSE._serialized_end=5738
  _PROJECTINFO._serialized_start=5740
  _PROJECTINFO._serialized_end=5820
  _COLLECTIONINFO._serialized_start=5822
  _COLLECTIONINFO._serialized_end=5911
  _BRANCHINFO._serialized_start=5913
  _BRANCHINFO._serialized_end=5990
  _LISTPROJECTSREQUEST._serialized_start=5992
  _LISTPROJECTSREQUEST._serialized_end=6013
  _LISTPROJECTSRESPONSE._serialized_start=6015
  _LISTPROJECTSRESPONSE._serialized_end=6083
  _LISTCOLLECTIONSREQUEST._serialized_start=6085
  _LISTCOLLECTIONSREQUEST._serialized_end=6193
  _LISTCOLLECTIONSRESPONSE._serialized_start=6195
  _LISTCOLLECTIONSRESPONSE._serialized_end=6272
  _DESCRIBEDATABASEREQUEST._serialized_start=6274
  _DESCRIBEDATABASEREQUEST._serialized_end=6355
  _DESCRIBECOLLECTIONREQUEST._serialized_start=6358
  _DESCRIBECOLLECTIONREQUEST._serialized_end=6512
  _DESCRIBEDATABASERESPONSE._serialized_start=6515
  _DESCRIBEDATABASERESPONSE._serialized_end=6683
  _DESCRIBECOLLECTIONRESPONSE._serialized_start=6686
  _DESCRIBECOLLECTIONRESPONSE._serialized_end=6866
  _PROJECTDESCRIPTION._serialized_start=6868
  _PROJECTDESCRIPTION._serialized_end=6888
  _COLLECTIONDESCRIPTION._serialized_start=6891
  _COLLECTIONDESCRIPTION._serialized_end=7066
  _PROJECTMETADATA._serialized_start=7068
  _PROJECTMETADATA._serialized_end=7085
  _DATABASEMETADATA._serialized_start=7087
  _DATABASEMETADATA._serialized_end=7105
  _COLLECTIONMETADATA._serialized_start=7107
  _COLLECTIONMETADATA._serialized_end=7127
  _COLLECTIONINDEX._serialized_start=7129
  _COLLECTIONINDEX._serialized_end=7213
  _FIELD._serialized_start=7215
  _FIELD._serialized_end=7236
  _BUILDCOLLECTIONINDEXREQUEST._serialized_start=7238
  _BUILDCOLLECTIONINDEXREQUEST._serialized_end=7320
  _BUILDCOLLECTIONINDEXRESPONSE._serialized_start=7322
  _BUILDCOLLECTIONINDEXRESPONSE._serialized_end=7401
  _BRANCHMETADATA._serialized_start=7403
  _BRANCHMETADATA._serialized_end=7419
  _CREATEAPPKEYREQUEST._serialized_start=7421
  _CREATEAPPKEYREQUEST._serialized_end=7494
  _CREATEAPPKEYRESPONSE._serialized_start=7496
  _CREATEAPPKEYRESPONSE._serialized_end=7566
  _UPDATEAPPKEYREQUEST._serialized_start=7568
  _UPDATEAPPKEYREQUEST._serialized_end=7653
  _UPDATEAPPKEYRESPONSE._serialized_start=7655
  _UPDATEAPPKEYRESPONSE._serialized_end=7725
  _APPKEY._serialized_start=7728
  _APPKEY._serialized_end=7896
  _LISTAPPKEYSREQUEST._serialized_start=7898
  _LISTAPPKEYSREQUEST._serialized_end=7935
  _LISTAPPKEYSRESPONSE._serialized_start=7937
  _LISTAPPKEYSRESPONSE._serialized_end=7999
  _DELETEAPPKEYREQUEST._serialized_start=8001
  _DELETEAPPKEYREQUEST._serialized_end=8051
  _DELETEAPPKEYRESPONSE._serialized_start=8053
  _DELETEAPPKEYRESPONSE._serialized_end=8092
  _ROTATEAPPKEYREQUEST._serialized_start=8094
  _ROTATEAPPKEYREQUEST._serialized_end=8144
  _ROTATEAPPKEYRESPONSE._serialized_start=8146
  _ROTATEAPPKEYRESPONSE._serialized_end=8208
  _TIGRIS._serialized_start=8211
  _TIGRIS._serialized_end=13901
# @@protoc_insertion_point(module_scope)



from ...google.api import annotations_pb2 as _annotations_pb2
from ...openapiv3 import annotations_pb2 as _annotations_pb2_1
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

ABORTED: Code
ALL: TigrisOperation
ALREADY_EXISTS: Code
AVG: MetricQuerySpaceAggregation
BAD_GATEWAY: Code
CANCELLED: Code
CONFLICT: Code
CONTENT_TOO_LARGE: Code
COUNT: MetricQueryFunction
DATA_LOSS: Code
DEADLINE_EXCEEDED: Code
DESCRIPTOR: _descriptor.FileDescriptor
FAILED_PRECONDITION: Code
INTERNAL: Code
INVALID_ARGUMENT: Code
MAX: MetricQuerySpaceAggregation
METADATA: TigrisOperation
METHOD_NOT_ALLOWED: Code
MIN: MetricQuerySpaceAggregation
NONE: MetricQueryFunction
NOT_FOUND: Code
OK: Code
OUT_OF_RANGE: Code
PERMISSION_DENIED: Code
RATE: MetricQueryFunction
READ: TigrisOperation
RESOURCE_EXHAUSTED: Code
ROLLUP_AGGREGATOR_AVG: RollupAggregator
ROLLUP_AGGREGATOR_COUNT: RollupAggregator
ROLLUP_AGGREGATOR_MAX: RollupAggregator
ROLLUP_AGGREGATOR_MIN: RollupAggregator
ROLLUP_AGGREGATOR_SUM: RollupAggregator
SUM: MetricQuerySpaceAggregation
UNAUTHENTICATED: Code
UNAVAILABLE: Code
UNIMPLEMENTED: Code
UNKNOWN: Code
WRITE: TigrisOperation

class AdditionalFunction(_message.Message):
    __slots__ = ["rollup"]
    ROLLUP_FIELD_NUMBER: _ClassVar[int]
    rollup: RollupFunction
    def __init__(self, rollup: _Optional[_Union[RollupFunction, _Mapping]] = ...) -> None: ...

class DataPoint(_message.Message):
    __slots__ = ["timestamp", "value"]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    timestamp: int
    value: float
    def __init__(self, timestamp: _Optional[int] = ..., value: _Optional[float] = ...) -> None: ...

class Error(_message.Message):
    __slots__ = ["code", "message"]
    CODE_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    code: Code
    message: str
    def __init__(self, code: _Optional[_Union[Code, str]] = ..., message: _Optional[str] = ...) -> None: ...

class ErrorDetails(_message.Message):
    __slots__ = ["code", "message", "retry"]
    CODE_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    RETRY_FIELD_NUMBER: _ClassVar[int]
    code: str
    message: str
    retry: RetryInfo
    def __init__(self, code: _Optional[str] = ..., message: _Optional[str] = ..., retry: _Optional[_Union[RetryInfo, _Mapping]] = ...) -> None: ...

class GetInfoRequest(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class GetInfoResponse(_message.Message):
    __slots__ = ["error", "server_version"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    SERVER_VERSION_FIELD_NUMBER: _ClassVar[int]
    error: Error
    server_version: str
    def __init__(self, server_version: _Optional[str] = ..., error: _Optional[_Union[Error, _Mapping]] = ...) -> None: ...

class MetricSeries(_message.Message):
    __slots__ = ["dataPoints", "metric", "scope", "to"]
    DATAPOINTS_FIELD_NUMBER: _ClassVar[int]
    FROM_FIELD_NUMBER: _ClassVar[int]
    METRIC_FIELD_NUMBER: _ClassVar[int]
    SCOPE_FIELD_NUMBER: _ClassVar[int]
    TO_FIELD_NUMBER: _ClassVar[int]
    dataPoints: _containers.RepeatedCompositeFieldContainer[DataPoint]
    metric: str
    scope: str
    to: int
    def __init__(self, to: _Optional[int] = ..., metric: _Optional[str] = ..., scope: _Optional[str] = ..., dataPoints: _Optional[_Iterable[_Union[DataPoint, _Mapping]]] = ..., **kwargs) -> None: ...

class QueryTimeSeriesMetricsRequest(_message.Message):
    __slots__ = ["additionalFunctions", "branch", "collection", "db", "function", "metric_name", "quantile", "space_aggregated_by", "space_aggregation", "tigris_operation", "to"]
    ADDITIONALFUNCTIONS_FIELD_NUMBER: _ClassVar[int]
    BRANCH_FIELD_NUMBER: _ClassVar[int]
    COLLECTION_FIELD_NUMBER: _ClassVar[int]
    DB_FIELD_NUMBER: _ClassVar[int]
    FROM_FIELD_NUMBER: _ClassVar[int]
    FUNCTION_FIELD_NUMBER: _ClassVar[int]
    METRIC_NAME_FIELD_NUMBER: _ClassVar[int]
    QUANTILE_FIELD_NUMBER: _ClassVar[int]
    SPACE_AGGREGATED_BY_FIELD_NUMBER: _ClassVar[int]
    SPACE_AGGREGATION_FIELD_NUMBER: _ClassVar[int]
    TIGRIS_OPERATION_FIELD_NUMBER: _ClassVar[int]
    TO_FIELD_NUMBER: _ClassVar[int]
    additionalFunctions: _containers.RepeatedCompositeFieldContainer[AdditionalFunction]
    branch: str
    collection: str
    db: str
    function: MetricQueryFunction
    metric_name: str
    quantile: float
    space_aggregated_by: _containers.RepeatedScalarFieldContainer[str]
    space_aggregation: MetricQuerySpaceAggregation
    tigris_operation: TigrisOperation
    to: int
    def __init__(self, db: _Optional[str] = ..., branch: _Optional[str] = ..., collection: _Optional[str] = ..., to: _Optional[int] = ..., metric_name: _Optional[str] = ..., tigris_operation: _Optional[_Union[TigrisOperation, str]] = ..., space_aggregation: _Optional[_Union[MetricQuerySpaceAggregation, str]] = ..., space_aggregated_by: _Optional[_Iterable[str]] = ..., function: _Optional[_Union[MetricQueryFunction, str]] = ..., quantile: _Optional[float] = ..., additionalFunctions: _Optional[_Iterable[_Union[AdditionalFunction, _Mapping]]] = ..., **kwargs) -> None: ...

class QueryTimeSeriesMetricsResponse(_message.Message):
    __slots__ = ["query", "series", "to"]
    FROM_FIELD_NUMBER: _ClassVar[int]
    QUERY_FIELD_NUMBER: _ClassVar[int]
    SERIES_FIELD_NUMBER: _ClassVar[int]
    TO_FIELD_NUMBER: _ClassVar[int]
    query: str
    series: _containers.RepeatedCompositeFieldContainer[MetricSeries]
    to: int
    def __init__(self, to: _Optional[int] = ..., query: _Optional[str] = ..., series: _Optional[_Iterable[_Union[MetricSeries, _Mapping]]] = ..., **kwargs) -> None: ...

class QuotaLimitsRequest(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class QuotaLimitsResponse(_message.Message):
    __slots__ = ["ReadUnits", "StorageSize", "WriteUnits"]
    READUNITS_FIELD_NUMBER: _ClassVar[int]
    ReadUnits: int
    STORAGESIZE_FIELD_NUMBER: _ClassVar[int]
    StorageSize: int
    WRITEUNITS_FIELD_NUMBER: _ClassVar[int]
    WriteUnits: int
    def __init__(self, ReadUnits: _Optional[int] = ..., WriteUnits: _Optional[int] = ..., StorageSize: _Optional[int] = ...) -> None: ...

class QuotaUsageRequest(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class QuotaUsageResponse(_message.Message):
    __slots__ = ["ReadUnits", "ReadUnitsThrottled", "StorageSize", "StorageSizeThrottled", "WriteUnits", "WriteUnitsThrottled"]
    READUNITSTHROTTLED_FIELD_NUMBER: _ClassVar[int]
    READUNITS_FIELD_NUMBER: _ClassVar[int]
    ReadUnits: int
    ReadUnitsThrottled: int
    STORAGESIZETHROTTLED_FIELD_NUMBER: _ClassVar[int]
    STORAGESIZE_FIELD_NUMBER: _ClassVar[int]
    StorageSize: int
    StorageSizeThrottled: int
    WRITEUNITSTHROTTLED_FIELD_NUMBER: _ClassVar[int]
    WRITEUNITS_FIELD_NUMBER: _ClassVar[int]
    WriteUnits: int
    WriteUnitsThrottled: int
    def __init__(self, ReadUnits: _Optional[int] = ..., WriteUnits: _Optional[int] = ..., StorageSize: _Optional[int] = ..., ReadUnitsThrottled: _Optional[int] = ..., WriteUnitsThrottled: _Optional[int] = ..., StorageSizeThrottled: _Optional[int] = ...) -> None: ...

class RetryInfo(_message.Message):
    __slots__ = ["delay"]
    DELAY_FIELD_NUMBER: _ClassVar[int]
    delay: int
    def __init__(self, delay: _Optional[int] = ...) -> None: ...

class RollupFunction(_message.Message):
    __slots__ = ["aggregator", "interval"]
    AGGREGATOR_FIELD_NUMBER: _ClassVar[int]
    INTERVAL_FIELD_NUMBER: _ClassVar[int]
    aggregator: RollupAggregator
    interval: int
    def __init__(self, aggregator: _Optional[_Union[RollupAggregator, str]] = ..., interval: _Optional[int] = ...) -> None: ...

class Code(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class TigrisOperation(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class MetricQueryFunction(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class RollupAggregator(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class MetricQuerySpaceAggregation(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from dataclasses import InitVar, dataclass, field
from datetime import datetime, timezone
from typing import Dict, List, Optional, Union

from api.generated.server.v1.api_pb2 import FacetCount as ProtoFacetCount
from api.generated.server.v1.api_pb2 import FacetStats as ProtoFacetStats
from api.generated.server.v1.api_pb2 import GroupedSearchHits as ProtoGroupedHits
from api.generated.server.v1.api_pb2 import Match as ProtoMatch
from api.generated.server.v1.api_pb2 import Page as ProtoPage
from api.generated.server.v1.api_pb2 import SearchFacet as ProtoSearchFacet
from api.generated.server.v1.api_pb2 import SearchHit as ProtoSearchHit
from api.generated.server.v1.api_pb2 import SearchHitMeta as ProtoSearchHitMeta
from api.generated.server.v1.api_pb2 import SearchMetadata as ProtoSearchMeta
from api.generated.server.v1.search_pb2 import DocStatus as ProtoDocStatus
from api.generated.server.v1.search_pb2 import (
    SearchIndexRequest as ProtoSearchIndexRequest,
)
from api.generated.server.v1.search_pb2 import (
    SearchIndexResponse as ProtoSearchIndexResponse,
)
from tigrisdb.errors import TigrisException
from tigrisdb.types import Document, Serializable
from tigrisdb.types.filters import Filter
from tigrisdb.types.sort import Sort
from tigrisdb.utils import marshal, unmarshal

dataclass_default_proto_field = field(
    default=None, repr=False, compare=False, hash=False
)


@dataclass
class FacetSize(Serializable):
    field: str
    size: int = 10

    def query(self):
        return {"size": self.size, "type": "value"}


FacetField = Union[str, FacetSize]


@dataclass
class VectorField(Serializable):
    field: str
    vector: List[float]

    def query(self):
        return {self.field: self.vector}


@dataclass
class Query:
    q: str = ""
    search_fields: List[str] = field(default_factory=list)
    vector_query: VectorField = None
    filter_by: Optional[Filter] = None
    facet_by: Union[str, List[FacetField]] = field(default_factory=list)
    sort_by: Union[Sort, List[Sort]] = field(default_factory=list)
    group_by: Union[str, List[str]] = field(default_factory=list)
    include_fields: List[str] = field(default_factory=list)
    exclude_fields: List[str] = field(default_factory=list)
    hits_per_page: int = 20

    def __build__(self, req: ProtoSearchIndexRequest):
        req.q = self.q or ""
        if self.search_fields:
            req.search_fields.extend(self.search_fields)
        if self.vector_query:
            req.vector = marshal(self.vector_query.query())
        if self.filter_by:
            req.filter = marshal(self.filter_by.query())
        if self.facet_by:
            f = {}
            if isinstance(self.facet_by, str):
                f[self.facet_by] = FacetSize(self.facet_by).query()
            elif isinstance(self.facet_by, list):
                for facet in self.facet_by:
                    if isinstance(facet, str):
                        f[facet] = FacetSize(facet).query()
                    elif isinstance(facet, FacetSize):
                        f[facet.field] = facet.query()
            req.facet = marshal(f)
        if self.sort_by:
            order = []
            if isinstance(self.sort_by, Sort):
                order.append(self.sort_by.query())
            elif isinstance(self.sort_by, list):
                order = [s.query() for s in self.sort_by]
            req.sort = marshal(order)
        if self.group_by:
            g = [self.group_by] if isinstance(self.group_by, str) else self.group_by
            req.group_by = marshal(g)
        if self.include_fields:
            req.include_fields.extend(self.include_fields)
        if self.exclude_fields:
            req.exclude_fields.extend(self.exclude_fields)
        req.page_size = self.hits_per_page


# Search result fields


@dataclass
class DocStatus:
    id: str = None
    error: Optional[TigrisException] = None
    _p: InitVar[ProtoDocStatus] = dataclass_default_proto_field

    def __post_init__(self, _p: ProtoDocStatus):
        if _p:
            if _p.HasField("error"):
                self.error = TigrisException(_p.error.message)
            if _p.id:
                self.id = _p.id


@dataclass
class TextMatchInfo:
    score: Optional[str] = None
    vector_distance: Optional[float] = None
    fields: List[str] = field(default_factory=list)
    _p: InitVar[ProtoMatch] = dataclass_default_proto_field

    def __post_init__(self, _p: ProtoMatch):
        if _p:
            self.fields = [f.name for f in _p.fields]
            if _p.HasField("vector_distance"):
                self.vector_distance = _p.vector_distance
            if _p.score:
                self.score = _p.score


@dataclass
class DocMeta:
    created_at: Optional[datetime] = None
    updated_at: Optional[datetime] = None
    text_match: TextMatchInfo = field(default_factory=TextMatchInfo)
    _p: InitVar[ProtoSearchHitMeta] = dataclass_default_proto_field

    def __post_init__(self, _p: ProtoSearchHitMeta):
        if _p:
            if _p.HasField("created_at"):
                self.created_at = _p.created_at.ToDatetime(tzinfo=timezone.utc)
            if _p.HasField("updated_at"):
                self.updated_at = _p.updated_at.ToDatetime(tzinfo=timezone.utc)
            if _p.HasField("match"):
                self.text_match = TextMatchInfo(_p=_p.match)


@dataclass
class IndexedDoc:
    doc: Optional[Document] = None
    meta: DocMeta = field(default_factory=DocMeta)
    _p: InitVar[ProtoSearchHit] = dataclass_default_proto_field

    def __post_init__(self, _p: ProtoSearchHit):
        if _p:
            if _p.data:
                self.doc = unmarshal(_p.data)
            if _p.HasField("metadata"):
                self.meta = DocMeta(_p=_p.metadata)


@dataclass
class GroupedHits:
    keys: [str] = field(default_factory=list)
    hits: [IndexedDoc] = field(default_factory=list)
    _p: InitVar[ProtoGroupedHits] = dataclass_default_proto_field

    def __post_init__(self, _p: ProtoGroupedHits):
        if _p:
            self.keys = _p.group_keys
            self.hits = [IndexedDoc(_p=h) for h in _p.hits]


@dataclass
class FacetCount:
    value: Optional[str] = None
    count: Optional[int] = None
    _p: InitVar[ProtoFacetCount] = dataclass_default_proto_field

    def __post_init__(self, _p: ProtoFacetCount):
        if _p:
            self.value = _p.value
            self.count = _p.count


@dataclass
class FacetStats:
    count: int = 0
    sum: Optional[float] = None
    avg: Optional[float] = None
    max: Optional[float] = None
    min: Optional[float] = None
    _p: InitVar[ProtoFacetStats] = dataclass_default_proto_field

    def __post_init__(self, _p: ProtoFacetStats):
        if _p:
            self.count = _p.count
            if _p.HasField("sum"):
                self.sum = _p.sum
            if _p.HasField("avg"):
                self.avg = _p.avg
            if _p.HasField("max"):
                self.max = _p.max
            if _p.HasField("min"):
                self.min = _p.min


@dataclass
class Facets:
    counts: [FacetCount] = field(default_factory=list)
    stats: FacetStats = field(default_factory=FacetStats)
    _p: InitVar[ProtoSearchFacet] = dataclass_default_proto_field

    def __post_init__(self, _p: ProtoSearchFacet):
        if _p:
            if _p.HasField("stats"):
                self.stats = FacetStats(_p=_p.stats)
            self.counts = [FacetCount(_p=fc) for fc in _p.counts]


@dataclass
class Page:
    current: int = 1
    size: int = 20
    _p: InitVar[ProtoPage] = dataclass_default_proto_field

    def __post_init__(self, _p: ProtoPage):
        if _p:
            if _p.current:
                self.current = _p.current
            self.size = _p.size


@dataclass
class Meta:
    found: int = 0
    total_pages: int = 1
    page: Page = field(default_factory=Page)
    _p: InitVar[ProtoSearchMeta] = dataclass_default_proto_field

    def __post_init__(self, _p: ProtoSearchMeta):
        if _p:
            if _p.HasField("page"):
                self.page = Page(_p=_p.page)
            self.found = _p.found
            if _p.total_pages:
                self.total_pages = _p.total_pages


@dataclass
class Result:
    hits: List[IndexedDoc] = field(default_factory=list)
    facets: Dict[str, Facets] = field(default_factory=dict)
    meta: Meta = field(default_factory=Meta)
    grouped_hits: [GroupedHits] = field(default_factory=list)
    _p: InitVar[ProtoSearchIndexResponse] = dataclass_default_proto_field

    def __post_init__(self, _p: ProtoSearchIndexResponse):
        if _p:
            self.hits = [IndexedDoc(_p=h) for h in _p.hits]
            self.grouped_hits = [GroupedHits(_p=g) for g in _p.group]
            self.facets = {k: Facets(_p=v) for k, v in _p.facets.items()}
            if _p.HasField("meta"):
                self.meta = Meta(_p=_p.meta)



import abc
from typing import Any, Dict

from tigrisdb.types import BaseOperator, Serializable


class SelectorFilter(Serializable, BaseOperator, abc.ABC):
    def __init__(self, field: str, value: Any):
        self.field = field
        self.value = value

    def query(self) -> Dict:
        return {self.field: {self.operator: self.value}}


class Eq(SelectorFilter):
    @property
    def operator(self):
        return ""

    def query(self) -> Dict:
        return {self.field: self.value}


class Not(SelectorFilter):
    @property
    def operator(self):
        return "$not"


class GT(SelectorFilter):
    @property
    def operator(self):
        return "$gt"


class GTE(SelectorFilter):
    @property
    def operator(self):
        return "$gte"


class LT(SelectorFilter):
    @property
    def operator(self):
        return "$lt"


class LTE(SelectorFilter):
    @property
    def operator(self):
        return "$lte"


class Regex(SelectorFilter):
    @property
    def operator(self):
        return "$regex"


class Contains(SelectorFilter):
    @property
    def operator(self):
        return "$contains"



import abc

from tigrisdb.types import BaseOperator, Serializable


class Sort(Serializable, BaseOperator, abc.ABC):
    field = ""

    def query(self):
        return {self.field: self.operator}


class Ascending(Sort):
    def __init__(self, field):
        self.field = field

    @property
    def operator(self):
        return "$asc"


class Descending(Sort):
    def __init__(self, field):
        self.field = field

    @property
    def operator(self):
        return "$desc"



from typing import cast

import grpc


class TigrisException(Exception):
    """Base class for all TigrisExceptions."""

    msg: str

    def __init__(self, msg: str, **kwargs):
        self.msg = msg
        kwargs["message"] = msg
        super(TigrisException, self).__init__(kwargs)


# TODO: make this typesafe
class TigrisServerError(TigrisException):
    def __init__(self, msg: str, e: grpc.RpcError):
        if isinstance(e.code(), grpc.StatusCode):
            self.code = cast(grpc.StatusCode, e.code())
        else:
            self.code = grpc.StatusCode.UNKNOWN

        self.details = e.details()
        super(TigrisServerError, self).__init__(
            msg, code=self.code.name, details=self.details
        )
        self.__suppress_context__ = True
